# Generated by Django 2.2.13 on 2020-06-18 16:21

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("standingsrequests", "0002_contactlabel_labelid_bigintegerfield"),
    ]

    operations = [
        migrations.RenameField(
            model_name="abstractstandingsrequest",
            old_name="actionBy",
            new_name="action_by",
        ),
        migrations.RenameField(
            model_name="abstractstandingsrequest",
            old_name="actionDate",
            new_name="action_date",
        ),
        migrations.RenameField(
            model_name="abstractstandingsrequest",
            old_name="contactID",
            new_name="contact_id",
        ),
        migrations.RenameField(
            model_name="abstractstandingsrequest",
            old_name="contactType",
            new_name="contact_type_id",
        ),
        migrations.RenameField(
            model_name="abstractstandingsrequest",
            old_name="effectiveDate",
            new_name="effective_date",
        ),
        migrations.RenameField(
            model_name="abstractstandingsrequest",
            old_name="effective",
            new_name="is_effective",
        ),
        migrations.RenameField(
            model_name="abstractstandingsrequest",
            old_name="requestDate",
            new_name="request_date",
        ),
        migrations.RenameField(
            model_name="alliancestanding", old_name="contactID", new_name="contact_id",
        ),
        migrations.RenameField(
            model_name="alliancestanding", old_name="set", new_name="contact_set",
        ),
        migrations.RenameField(
            model_name="contactlabel", old_name="set", new_name="contact_set",
        ),
        migrations.RenameField(
            model_name="contactlabel", old_name="labelID", new_name="label_id",
        ),
        migrations.RenameField(
            model_name="corpstanding", old_name="contactID", new_name="contact_id",
        ),
        migrations.RenameField(
            model_name="corpstanding", old_name="set", new_name="contact_set",
        ),
        migrations.RenameField(
            model_name="evenamecache", old_name="entityID", new_name="entity_id",
        ),
        migrations.RenameField(
            model_name="pilotstanding", old_name="contactID", new_name="contact_id",
        ),
        migrations.RenameField(
            model_name="pilotstanding", old_name="set", new_name="contact_set",
        ),
        migrations.RenameField(
            model_name="pilotstanding", old_name="inWatchlist", new_name="is_watched",
        ),
    ]



# Generated by Django 2.2.13 on 2020-07-07 19:43

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    atomic = False

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("standingsrequests", "0004_fix_types_and_indices"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="AllianceStanding", new_name="AllianceContact",
        ),
        migrations.RenameModel(old_name="PilotStanding", new_name="CharacterContact",),
        migrations.RenameModel(old_name="CorpStanding", new_name="CorporationContact",),
        migrations.RenameModel(old_name="EveNameCache", new_name="EveEntity",),
        migrations.RenameModel(
            old_name="StandingsRequest", new_name="StandingRequest",
        ),
        migrations.RenameModel(
            old_name="StandingsRevocation", new_name="StandingRevocation",
        ),
    ]



from __future__ import unicode_literals
from django.conf.urls import url, include
from . import views

app_name = "standingsrequests"

local_urlpatterns = [
    url(r"^$", views.index_view, name="index"),
    url(r"^request_entities$", views.partial_request_entities, name="request_entities"),
    url(
        r"^request_pilot_standing/(?P<character_id>\d+)/",
        views.request_pilot_standing,
        name="request_pilot_standing",
    ),
    url(
        r"^remove_pilot_standing/(?P<character_id>\d+)/",
        views.remove_pilot_standing,
        name="remove_pilot_standing",
    ),
    url(
        r"^request_corp_standing/(?P<corporation_id>\d+)/",
        views.request_corp_standing,
        name="request_corp_standing",
    ),
    url(
        r"^remove_corp_standing/(?P<corporation_id>\d+)/",
        views.remove_corp_standing,
        name="remove_corp_standing",
    ),
    url(r"^view/pilots/$", views.view_pilots_standings, name="view_pilots"),
    url(
        r"^view/pilots/json/$",
        views.view_pilots_standings_json,
        name="view_pilots_json",
    ),
    url(
        r"^view/pilots/download/$",
        views.download_pilot_standings,
        name="download_pilots",
    ),
    url(r"^view/corps/$", views.view_groups_standings, name="view_groups"),
    url(
        r"^view/corps/json$", views.view_groups_standings_json, name="view_groups_json"
    ),
    url(r"^manage/$", views.manage_standings, name="manage"),
    url(
        r"^manage/requests/$",
        views.manage_get_requests_json,
        name="manage_get_requests_json",
    ),
    # Should always follow the path of the GET path above
    url(
        r"^manage/requests/(?P<contact_id>\d+)/$",
        views.manage_requests_write,
        name="manage_requests_write",
    ),
    url(
        r"^manage/revocations/$",
        views.manage_get_revocations_json,
        name="manage_get_revocations_json",
    ),
    url(
        r"^manage/revocations/(?P<contact_id>\d+)/$",
        views.manage_revocations_write,
        name="manage_revocations_write",
    ),
    url(r"^view/requests/$", views.view_active_requests, name="view_requests"),
    url(r"^view/requests/json/$", views.view_requests_json, name="view_requests_json",),
    url(r"^manage/setuptoken/$", views.view_auth_page, name="view_auth_page"),
    url(
        r"^requester_add_scopes/$",
        views.view_requester_add_scopes,
        name="view_requester_add_scopes",
    ),
]

urlpatterns = [
    url(r"^standingsrequests/", include((local_urlpatterns, "standingsrequests"))),
]



from datetime import timedelta

from celery import chain, shared_task

from django.contrib.auth.models import User
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _

from allianceauth.notifications import notify
from allianceauth.services.hooks import get_extension_logger

from . import __title__
from .app_settings import (
    SR_STANDINGS_STALE_HOURS,
    SR_SYNC_BLUE_ALTS_ENABLED,
)
from .helpers.viewcache import cache_view_pilots_json, cache_view_groups_json
from .models import (
    AllianceContact,
    CharacterAssociation,
    CharacterContact,
    ContactLabel,
    ContactSet,
    CorporationContact,
    StandingRequest,
    StandingRevocation,
)
from .utils import LoggerAddTag

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


@shared_task(name="standings_requests.update_all")
def update_all(user_pk: int = None):
    """Updates standings, names cache and associations"""
    my_chain = chain(
        [
            standings_update.si(),
            update_associations_auth.si(),
            update_associations_api.si(),
            report_result_to_user.si(user_pk),
        ]
    )
    my_chain.delay()


@shared_task(name="standings_requests.report_result_to_user")
def report_result_to_user(user_pk: int = None):
    if user_pk:
        try:
            user = User.objects.get(pk=user_pk)
        except User.DoesNotExist:
            logger.warning("Can not find a user with pk %d", user_pk)
            return
        else:
            source_entity = ContactSet.standings_source_entity()
            notify(
                user,
                _("%s: Standings loaded" % __title__),
                _(
                    "Standings have been successfully loaded for %s"
                    % source_entity.name
                ),
                level="success",
            )


@shared_task(name="standings_requests.standings_update")
def standings_update():
    """Updates standings from ESI"""
    logger.info("Standings API update started")
    contact_set = ContactSet.objects.create_new_from_api()
    if not contact_set:
        logger.warn(
            "Standings API update returned None (API error probably),"
            "aborting standings update"
        )
    else:
        if SR_SYNC_BLUE_ALTS_ENABLED:
            contact_set.generate_standing_requests_for_blue_alts()
        StandingRequest.objects.process_requests()
        StandingRevocation.objects.process_requests()
        cache_view_pilots_json.clear()
        cache_view_groups_json.clear()


@shared_task(name="standings_requests.validate_requests")
def validate_requests():
    logger.info("Validating standings request running")
    count = StandingRequest.objects.validate_requests()
    logger.info("Dealt with %d invalid standings requests", count)


@shared_task(name="standings_requests.update_associations_auth")
def update_associations_auth():
    """Update associations from local auth data (Main character, corporations)"""
    logger.info("Associations updating from Auth")
    CharacterAssociation.objects.update_from_auth()
    logger.info("Finished Associations update from Auth")


@shared_task(name="standings_requests.update_associations_api")
def update_associations_api():
    """Update character associations from the EVE API (corporations)"""
    logger.info("Associations updating from EVE API")
    CharacterAssociation.objects.update_from_api()
    logger.info("Finished associations update from EVE API")


@shared_task(name="standings_requests.purge_stale_data")
def purge_stale_data():
    """Delete all the data which is beyond its useful life. 
    There is no harm in disabling this if you wish to keep everything.
    """
    my_chain = chain([purge_stale_standings_data.si()])
    my_chain.delay()


@shared_task
def purge_stale_standings_data():
    """Deletes all stale (=older than threshold hours) contact sets 
    except the last remaining contact set
    """
    logger.info("Purging stale standings data")
    cutoff_date = now() - timedelta(hours=SR_STANDINGS_STALE_HOURS)
    try:
        latest_standings = ContactSet.objects.latest()
        stale_contacts_qs = ContactSet.objects.filter(date__lt=cutoff_date).exclude(
            id=latest_standings.id
        )
        if stale_contacts_qs.exists():
            logger.debug("Deleting old ContactSets")
            # we can't just do standigs.delete()
            # because with lots of them it uses lots of memory
            # lets go over them one by one and delete
            for contact_set in stale_contacts_qs:
                CharacterContact.objects.filter(contact_set=contact_set).delete()
                CorporationContact.objects.filter(contact_set=contact_set).delete()
                AllianceContact.objects.filter(contact_set=contact_set).delete()
                ContactLabel.objects.filter(contact_set=contact_set).delete()

            stale_contacts_qs.delete()
        else:
            logger.debug("No ContactSets to delete")

    except ContactSet.DoesNotExist:
        logger.warn("No ContactSets available, nothing to delete")



from django.apps import AppConfig
from . import __title__, __version__


class StandingsRequestsConfig(AppConfig):
    name = "standingsrequests"
    label = "standingsrequests"
    verbose_name = "%s v%s" % (__title__, __version__)

    def ready(self):
        pass



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-05-08 12:57
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="AbstractStandingsRequest",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("contactID", models.IntegerField()),
                ("contactType", models.IntegerField()),
                ("requestDate", models.DateTimeField(auto_now_add=True)),
                ("actionDate", models.DateTimeField(null=True)),
                ("effective", models.BooleanField(default=False)),
                ("effectiveDate", models.DateTimeField(null=True)),
            ],
            options={
                "permissions": (
                    ("affect_standings", "User can process standings requests."),
                    ("request_standings", "User can request standings."),
                ),
            },
        ),
        migrations.CreateModel(
            name="AllianceStanding",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("contactID", models.IntegerField()),
                ("name", models.CharField(max_length=254)),
                ("standing", models.FloatField()),
            ],
            options={"abstract": False,},
        ),
        migrations.CreateModel(
            name="CharacterAssociation",
            fields=[
                (
                    "character_id",
                    models.IntegerField(primary_key=True, serialize=False),
                ),
                ("corporation_id", models.IntegerField(null=True)),
                ("alliance_id", models.IntegerField(null=True)),
                ("main_character_id", models.IntegerField(null=True)),
                ("updated", models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name="ContactLabel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("labelID", models.IntegerField()),
                ("name", models.CharField(max_length=254)),
            ],
        ),
        migrations.CreateModel(
            name="ContactSet",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("date", models.DateTimeField(auto_now_add=True)),
                ("name", models.CharField(max_length=254)),
            ],
            options={
                "permissions": (
                    ("view", "User can view standings"),
                    ("download", "User can export standings to a CSV file"),
                ),
                "get_latest_by": "date",
            },
        ),
        migrations.CreateModel(
            name="CorpStanding",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("contactID", models.IntegerField()),
                ("name", models.CharField(max_length=254)),
                ("standing", models.FloatField()),
                ("labels", models.ManyToManyField(to="standingsrequests.ContactLabel")),
                (
                    "set",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="standingsrequests.ContactSet",
                    ),
                ),
            ],
            options={"abstract": False,},
        ),
        migrations.CreateModel(
            name="EveNameCache",
            fields=[
                ("entityID", models.IntegerField(primary_key=True, serialize=False)),
                ("name", models.CharField(max_length=254)),
                ("updated", models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name="PilotStanding",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("contactID", models.IntegerField()),
                ("name", models.CharField(max_length=254)),
                ("standing", models.FloatField()),
                ("inWatchlist", models.BooleanField(default=False)),
                ("labels", models.ManyToManyField(to="standingsrequests.ContactLabel")),
                (
                    "set",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="standingsrequests.ContactSet",
                    ),
                ),
            ],
            options={"abstract": False,},
        ),
        migrations.CreateModel(
            name="StandingsRequest",
            fields=[
                (
                    "abstractstandingsrequest_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="standingsrequests.AbstractStandingsRequest",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            bases=("standingsrequests.abstractstandingsrequest",),
        ),
        migrations.CreateModel(
            name="StandingsRevocation",
            fields=[
                (
                    "abstractstandingsrequest_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="standingsrequests.AbstractStandingsRequest",
                    ),
                ),
            ],
            bases=("standingsrequests.abstractstandingsrequest",),
        ),
        migrations.AddField(
            model_name="contactlabel",
            name="set",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="standingsrequests.ContactSet",
            ),
        ),
        migrations.AddField(
            model_name="alliancestanding",
            name="labels",
            field=models.ManyToManyField(to="standingsrequests.ContactLabel"),
        ),
        migrations.AddField(
            model_name="alliancestanding",
            name="set",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="standingsrequests.ContactSet",
            ),
        ),
        migrations.AddField(
            model_name="abstractstandingsrequest",
            name="actionBy",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]



"""helper for managing view caches accross modules"""

from django.core.cache import cache


class DataViewCache:
    """class for managing server side caching of data views"""

    CACHE_KEY_BASE = "STANDINGSREQUESTS_VIEW_CACHE"
    CACHE_DURATION = 600

    def __init__(self, view_name: str) -> None:
        self._view_name = str(view_name)

    @property
    def view_name(self) -> str:
        return self._view_name

    def get_or_set(self, func):
        return cache.get_or_set(self._cache_key(), func, timeout=self.CACHE_DURATION)

    def clear(self) -> None:
        cache.delete(self._cache_key())

    def _cache_key(self) -> str:
        return f"{self.CACHE_KEY_BASE}_{self.view_name}"


cache_view_pilots_json = DataViewCache("view_pilots_json")

cache_view_groups_json = DataViewCache("view_groups_json")
